@@ -1,6 +1,6 @@
 package Bundle::LWP;
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 
 1;
 
@@ -16,11 +16,15 @@ Bundle::LWP - A bundle to install all libwww-perl related modules
 
 =head1 CONTENTS
 
+URI 0.10           - There are URIs everywhere
+
 Net::FTP 2.00      - If you want ftp://-support
 
 MIME::Base64       - Used in authentication headers
 
-MD5                - Needed to Digest authentication
+Digest::MD5        - Needed to do Digest authentication
+
+HTML::HeadParser   - To get the correct $res->base
 
 LWP                - The reason why you need the modules above
 
