@@ -1,4 +1,4 @@
-# $Id: UserAgent.pm,v 1.61 1998/05/07 15:00:56 aas Exp $
+# $Id: UserAgent.pm,v 1.66 1999/03/20 07:37:36 gisle Exp $
 
 package LWP::UserAgent;
 use strict;
@@ -29,52 +29,52 @@ core of libwww-perl library. For simple uses this class can be used
 directly to dispatch WWW requests, alternatively it can be subclassed
 for application-specific behaviour.
 
-In normal usage the application creates a UserAgent object, and then
-configures it with values for timeouts proxies, name, etc. The next
-step is to create an instance of C<HTTP::Request> for the request that
+In normal use the application creates a UserAgent object, and then
+configures it with values for timeouts, proxies, name, etc. It next
+creates an instance of C<HTTP::Request> for the request that
 needs to be performed. This request is then passed to the UserAgent
 request() method, which dispatches it using the relevant protocol,
 and returns a C<HTTP::Response> object.
 
 The basic approach of the library is to use HTTP style communication
-for all protocol schemes, i.e. you will receive an C<HTTP::Response>
-object also for gopher or ftp requests.  In order to achieve even more
-similarities with HTTP style communications, gopher menus and file
-directories will be converted to HTML documents.
+for all protocol schemes, i.e. you also receive an C<HTTP::Response>
+object for gopher or ftp requests.  In order to achieve even more
+similarity to HTTP style communications, gopher menus and file
+directories are converted to HTML documents.
 
 The request() method can process the content of the response in one of
-three ways: in core, into a file, or into repeated calls of a
+three ways: in core, into a file, or into repeated calls to a
 subroutine.  You choose which one by the kind of value passed as the
 second argument to request().
 
-The in core variant simply returns the content in a scalar attribute
-called content() of the response object, and is suitable for small
+The in core variant simply stores the content in a scalar 'content' attribute
+of the response object and is suitable for small
 HTML replies that might need further parsing.  This variant is used if
 the second argument is missing (or is undef).
 
 The filename variant requires a scalar containing a filename as the
-second argument to request(), and is suitable for large WWW objects
-which need to be written directly to the file, without requiring large
+second argument to request() and is suitable for large WWW objects
+which need to be written directly to the file without requiring large
 amounts of memory. In this case the response object returned from
-request() will have empty content().  If the request fails, then the
-content() might not be empty, and the file will be untouched.
+request() will have an empty content attribute.  If the request fails, then the
+content might not be empty, and the file will be untouched.
 
 The subroutine variant requires a reference to callback routine as the
 second argument to request() and it can also take an optional chuck
-size as third argument.  This variant can be used to construct
+size as the third argument.  This variant can be used to construct
 "pipe-lined" processing, where processing of received chuncks can
 begin before the complete data has arrived.  The callback function is
 called with 3 arguments: the data received this time, a reference to
 the response object and a reference to the protocol object.  The
-response object returned from request() will have empty content().  If
-the request fails, then the the callback routine will not have been
-called, and the response->content() might not be empty.
+response object returned from request() will have empty content.  If
+the request fails, then the the callback routine is
+called, and the response->content might not be empty.
 
-The request can be aborted by calling die() within the callback
+The request can be aborted by calling die() in the callback
 routine.  The die message will be available as the "X-Died" special
 response header field.
 
-The library also accepts that you put a subroutine reference as
+The library also allows you to use a subroutine reference as
 content in the request object.  This subroutine should return the
 content (possibly in pieces) when called.  It should return an empty
 string when there is no more content.
@@ -92,13 +92,10 @@ use vars qw(@ISA $VERSION);
 
 require LWP::MemberMixin;
 @ISA = qw(LWP::MemberMixin);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.61 $ =~ /(\d+)\.(\d+)/);
-
-
-require URI::URL;
-require HTTP::Request;
-require HTTP::Response;
+$VERSION = sprintf("%d.%02d", q$Revision: 1.66 $ =~ /(\d+)\.(\d+)/);
 
+use HTTP::Request ();
+use HTTP::Response ();
 use HTTP::Date ();
 
 use LWP ();
@@ -107,9 +104,6 @@ use LWP::Protocol ();
 
 use Carp ();
 
-use AutoLoader ();
-*AUTOLOAD = \&AutoLoader::AUTOLOAD;  # import the AUTOLOAD method
-
 
 =item $ua = new LWP::UserAgent;
 
@@ -147,7 +141,7 @@ sub new
 This method dispatches a single WWW request on behalf of a user, and
 returns the response received.  The C<$request> should be a reference
 to a C<HTTP::Request> object with values defined for at least the
-method() and url() attributes.
+method() and uri() attributes.
 
 If C<$arg> is a scalar it is taken as a filename where the content of
 the response is stored.
@@ -173,6 +167,9 @@ sub simple_request
 	unless $method;
     return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST, "URL missing")
 	unless $url;
+    return HTTP::Response->new(&HTTP::Status::RC_BAD_REQUEST, "URL must be absolute")
+	unless $url->scheme;
+	
 
     LWP::Debug::trace("$method $url");
 
@@ -237,7 +234,7 @@ sub simple_request
 =item $ua->request($request, $arg [, $size])
 
 Process a request, including redirects and security.  This method may
-actually send several different simple reqeusts.
+actually send several different simple requests.
 
 The arguments are the same as for C<simple_request()>.
 
@@ -263,10 +260,15 @@ sub request
 	my $referral = $request->clone;
 
 	# And then we update the URL based on the Location:-header.
-	# Some servers erroneously return a relative URL for redirects,
-	# so make it absolute if it not already is.
-	my $referral_uri = (URI::URL->new($response->header('Location'),
-					  $response->base))->abs(undef,1);
+	my $referral_uri = $response->header('Location');
+	{
+	    # Some servers erroneously return a relative URL for redirects,
+	    # so make it absolute if it not already is.
+	    local $URI::ABS_ALLOW_RELATIVE_SCHEME = 1;
+	    my $base = $response->base;
+	    $referral_uri = $HTTP::URI_CLASS->new($referral_uri, $base)
+		            ->abs($base);
+	}
 
 	$referral->url($referral_uri);
 
@@ -306,6 +308,9 @@ sub request
 	my $scheme = lc(shift(@$challenge));
 	shift(@$challenge); # no value
 	$challenge = { @$challenge };  # make rest into a hash
+	for (keys %$challenge) {       # make sure all keys are lower case
+	    $challenge->{lc $_} = delete $challenge->{$_};
+	}
 
 	unless ($scheme =~ /^([a-z]+(?:-[a-z]+)*)$/) {
 	    $response->header("Client-Warning" => 
@@ -340,7 +345,7 @@ sub request
 =item $ua->redirect_ok
 
 This method is called by request() before it tries to do any
-redirects.  It should return a true value if the redirect is allowed
+redirects.  It should return a true value if a redirect is allowed
 to be performed. Subclasses might want to override this.
 
 The default implementation will return FALSE for POST request and TRUE
@@ -397,9 +402,9 @@ sub get_basic_credentials
     my($self, $realm, $uri, $proxy) = @_;
     return if $proxy;
 
-    my $netloc = $uri->netloc;
-    if (exists $self->{'basic_authentication'}{$netloc}{$realm}) {
-	return @{ $self->{'basic_authentication'}{$netloc}{$realm} };
+    my $host_port = $uri->host_port;
+    if (exists $self->{'basic_authentication'}{$host_port}{$realm}) {
+	return @{ $self->{'basic_authentication'}{$host_port}{$realm} };
     }
 
     return (undef, undef);
@@ -449,7 +454,7 @@ TRUE.  Do not turn this off, unless you know what you are doing.
 =item $ua->max_size([$bytes])
 
 Get/set the size limit for response content.  The default is undef,
-which means that there is not limit.  If the returned response content
+which means that there is no limit.  If the returned response content
 is only partial, because the size limit was exceeded, then a
 "X-Content-Range" header will be added to the response.
 
@@ -472,20 +477,6 @@ sub use_alarm
 }
 
 
-# Declarations of AutoLoaded methods
-sub clone;
-sub is_protocol_supported;
-sub mirror;
-sub proxy;
-sub env_proxy;
-sub no_proxy;
-sub _need_proxy;
-
-
-1;
-__END__
-
-
 =item $ua->clone;
 
 Returns a copy of the LWP::UserAgent object
@@ -509,7 +500,7 @@ sub clone
 
 You can use this method to query if the library currently support the
 specified C<scheme>.  The C<scheme> might be a string (like 'http' or
-'ftp') or it might be an URI::URL object reference.
+'ftp') or it might be an URI object reference.
 
 =cut
 
@@ -517,13 +508,14 @@ sub is_protocol_supported
 {
     my($self, $scheme) = @_;
     if (ref $scheme) {
-	# assume we got a reference to an URI::URL object
-	$scheme = $scheme->abs->scheme;
+	# assume we got a reference to an URI object
+	$scheme = $scheme->scheme;
     } else {
 	Carp::croak("Illeal scheme '$scheme' passed to is_protocol_supported")
 	    if $scheme =~ /\W/;
 	$scheme = lc $scheme;
     }
+    local($SIG{__DIE__});  # protect agains user defined die handlers
     return LWP::Protocol::implementor($scheme);
 }
 
@@ -677,35 +669,23 @@ sub no_proxy {
 sub _need_proxy
 {
     my($self, $url) = @_;
+    $url = $HTTP::URI_CLASS->new($url) unless ref $url;
 
-    $url = new URI::URL($url) unless ref $url;
-
-    LWP::Debug::trace("($url)");
-
-    # check the list of noproxies
-
-    if (@{ $self->{'no_proxy'} }) {
-	my $host = $url->host;
-	return undef unless defined $host;
-	my $domain;
-	for $domain (@{ $self->{'no_proxy'} }) {
-	    if ($host =~ /$domain$/) {
-		LWP::Debug::trace("no_proxy configured");
-		return undef;
+    my $scheme = $url->scheme || return;
+    if (my $proxy = $self->{'proxy'}{$scheme}) {
+	if (@{ $self->{'no_proxy'} }) {
+	    if (my $host = eval { $url->host }) {
+		for my $domain (@{ $self->{'no_proxy'} }) {
+		    if ($host =~ /\Q$domain\E$/) {
+			LWP::Debug::trace("no_proxy configured");
+			return;
+		    }
+		}
 	    }
 	}
+	LWP::Debug::debug("Proxied to $proxy");
+	return $HTTP::URI_CLASS->new($proxy);
     }
-
-    # Currently configured per scheme.
-    # Eventually want finer granularity
-
-    my $scheme = $url->scheme;
-    if (exists $self->{'proxy'}{$scheme}) {
-
-	LWP::Debug::debug('Proxied');
-	return new URI::URL($self->{'proxy'}{$scheme});
-    }
-
     LWP::Debug::debug('Not proxied');
     undef;
 }
@@ -721,7 +701,7 @@ F<lwp-mirror> for examples of usage.
 
 =head1 COPYRIGHT
 
-Copyright 1995-1997 Gisle Aas.
+Copyright 1995-1998 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
